// This Heap sort algorithm is modified slightly from Dr. Jonathan Mohr's original available at:
// http://www.augustana.ab.ca/~mohrj/courses/2004.winter/csc310/source/HeapSort.java.html

public class HeapSort {
	private int[] a;
	
	// constructor for sorted array accepts array input
    public HeapSort(int[] anArray)
    {
       a = anArray;
    }

    // top level sort method
    public void sort()
    {
       sort( a.length - 1 );
    }
 
    // heap sort method
    public void sort( int end )
    {
       // establish the heap in place ("heapify" the array)
       for ( int i = end / 2; i >= 1; i-- )
          siftDown( i, end, a[i] );
       
       // place the largest element last,
       // 2nd largest 2nd last and so on
       for ( int i = end; i > 1; i-- )
       {
          // a[1] is always the next-biggest element
          swap( 1, i );
 
          // shrink heap by 1 element
          siftDown( 1, i - 1, a[1] );
       }
    }
 

    private void siftDown( int root, int end, int key )
    {
       int child = 2 * root; // start with left child
       
       // Find the larger child.
       if ( child < end && a[child] < a[child + 1] )
          child++;  // point to right child if it is larger than left
 
       // If the larger child is larger than the root, move the larger child to the root and sift
       // the former root element down.
       if ( child <= end && key < a[child] )
       {
          a[root] = a[child];
          siftDown( child, end, key );
       }
       else
          a[root] = key;
    }

   // method for swapping 2 elements
   private void swap(int i, int j)
    {
       int temp = a[i];
       a[i] = a[j];
       a[j] = temp;
    }
 }

